@@ -2184,6 +2184,11 @@ int main (int argc, char **argv)
 	}
 #endif
 
+	if (gtk_clutter_init (NULL, NULL) != CLUTTER_INIT_SUCCESS) {
+		g_warning ("Could not initialise clutter-gtk");
+		return 1;
+	}
+
 	context = g_option_context_new ("- Play audio and video inside a web browser");
 	baconoptiongroup = bacon_video_widget_get_option_group();
 	g_option_context_add_main_entries (context, option_entries, GETTEXT_PACKAGE);
@@ -2255,9 +2260,15 @@ int main (int argc, char **argv)
 
 	emb = g_object_new (TOTEM_TYPE_EMBEDDED,
 			    "application-id", svcname,
-			    "flags", G_APPLICATION_FLAGS_NONE,
+			    "flags", G_APPLICATION_NON_UNIQUE,
 			    NULL);
 
+	if (g_application_register (G_APPLICATION (emb), NULL, &e) == FALSE) {
+		g_warning ("Failed to register application: %s", e->message);
+		g_error_free (e);
+		return 1;
+	}
+
 	emb->state = TOTEM_STATE_INVALID;
 	emb->width = -1;
 	emb->height = -1;
