@@ -70,6 +70,7 @@
 	"</ui>"
 
 #define BROWSE_FLAGS          (GRL_RESOLVE_FAST_ONLY | GRL_RESOLVE_IDLE_RELAY)
+#define RESOLVE_FLAGS 	      (GRL_RESOLVE_FULL | GRL_RESOLVE_IDLE_RELAY)
 #define PAGE_SIZE             50
 #define THUMB_SEARCH_SIZE     256
 #define THUMB_BROWSE_SIZE     32
@@ -570,16 +571,30 @@ play (TotemGriloPlugin *self,
 	}
 
 	/* If url is a slow key, then we need to full resolve it */
-	if (resolve_url) {
+	if (resolve_url &&
+	    grl_source_supported_operations (source) & GRL_OP_RESOLVE) {
 		const GList *slow_keys;
-		GList *url_keys;
+
 		slow_keys = grl_source_slow_keys (source);
+
 		if (g_list_find ((GList *) slow_keys, GINT_TO_POINTER (GRL_METADATA_KEY_URL)) != NULL) {
+			GList *url_keys;
+			GrlOperationOptions *resolve_options;
+
+			resolve_options = grl_operation_options_new (NULL);
+			grl_operation_options_set_flags (resolve_options, RESOLVE_FLAGS);
+
 			url_keys = grl_metadata_key_list_new (GRL_METADATA_KEY_URL, NULL);
-			grl_source_resolve (source, media, url_keys, 0, resolve_url_cb, self);
+			grl_source_resolve (source, media, url_keys, resolve_options, resolve_url_cb, self);
+
+			g_object_unref (resolve_options);
 			g_list_free (url_keys);
 			return;
 		}
+	} else if (resolve_url) {
+		/* If source does not support resolve() operation, then use the current media */
+		resolve_url_cb (source, 0, media, NULL, NULL);
+		return;
 	}
 
 	g_warning ("Current element has no URL to play");
@@ -642,15 +657,34 @@ search_cb (GrlSource *source,
 	}
 }
 
-static void
-search_more (TotemGriloPlugin *self)
+static GrlOperationOptions *
+get_search_options (TotemGriloPlugin *self)
 {
 	GrlOperationOptions *default_options;
+	GrlOperationOptions *supported_options;
 
 	default_options = grl_operation_options_new (NULL);
 	grl_operation_options_set_flags (default_options, BROWSE_FLAGS);
-	grl_operation_options_set_skip (default_options, (self->priv->search_page - 1) * PAGE_SIZE);
+	grl_operation_options_set_skip (default_options, self->priv->search_page * PAGE_SIZE);
 	grl_operation_options_set_count (default_options, PAGE_SIZE);
+	grl_operation_options_set_type_filter (default_options, GRL_TYPE_FILTER_VIDEO);
+
+	/* And now remove all the unsupported filters and options */
+	grl_operation_options_obey_caps (default_options,
+					 grl_source_get_caps (GRL_SOURCE (self->priv->search_source), GRL_OP_SEARCH),
+					 &supported_options,
+					 NULL);
+	g_object_unref (default_options);
+
+	return supported_options;
+}
+
+static void
+search_more (TotemGriloPlugin *self)
+{
+	GrlOperationOptions *search_options;
+
+	search_options = get_search_options (self);
 
 	gtk_widget_set_sensitive (self->priv->search_entry, FALSE);
 	self->priv->search_page++;
@@ -660,7 +694,7 @@ search_more (TotemGriloPlugin *self)
 			grl_source_search (self->priv->search_source,
 			                   self->priv->search_text,
 			                   search_keys (),
-			                   default_options,
+			                   search_options,
 			                   search_cb,
 			                   self);
 	} else {
@@ -668,11 +702,11 @@ search_more (TotemGriloPlugin *self)
 			grl_multiple_search (NULL,
 			                     self->priv->search_text,
 			                     search_keys (),
-			                     default_options,
+			                     search_options,
 			                     search_cb,
 			                     self);
 	}
-	g_object_unref (default_options);
+	g_object_unref (search_options);
 
 	if (self->priv->search_id == 0) {
 		search_cb (self->priv->search_source, 0, NULL, 0, self, NULL);
