@@ -146,6 +146,7 @@ enum {
 	TITLE_CUSTOM_COL,
 	SUBTITLE_URI_COL,
 	FILE_MONITOR_COL,
+	MOUNT_COL,
 	MIME_TYPE_COL,
 	NUM_COLS
 };
@@ -479,6 +480,8 @@ drop_finished_cb (TotemPlaylist *playlist, GAsyncResult *result, gpointer user_d
 {
 	totem_playlist_add_mrls_finish (playlist, result, NULL);
 
+	g_clear_pointer (&playlist->priv->tree_path, gtk_tree_path_free);
+
 	/* Emit the "changed" signal once the last dropped MRL has been added to the playlist */
 	g_signal_emit (G_OBJECT (playlist),
 	               totem_playlist_table_signals[CHANGED], 0,
@@ -487,12 +490,12 @@ drop_finished_cb (TotemPlaylist *playlist, GAsyncResult *result, gpointer user_d
 
 static void
 drop_cb (GtkWidget        *widget,
-         GdkDragContext   *context, 
+         GdkDragContext   *context,
 	 gint              x,
 	 gint              y,
-	 GtkSelectionData *data, 
-	 guint             info, 
-	 guint             _time, 
+	 GtkSelectionData *data,
+	 guint             info,
+	 guint             _time,
 	 TotemPlaylist    *playlist)
 {
 	char **list;
@@ -573,8 +576,6 @@ drop_cb (GtkWidget        *widget,
 	g_strfreev (list);
 	g_list_free (file_list);
 	gtk_drag_finish (context, TRUE, FALSE, _time);
-	gtk_tree_path_free (playlist->priv->tree_path);
-	playlist->priv->tree_path = NULL;
 }
 
 void
@@ -643,7 +644,7 @@ playlist_show_popup_menu (TotemPlaylist *playlist, GdkEventButton *event)
 	}
 
 	count = gtk_tree_selection_count_selected_rows (playlist->priv->selection);
-	
+
 	if (count == 0) {
 		return FALSE;
 	}
@@ -729,9 +730,9 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 	}
 }
 
-static gboolean 
+static gboolean
 button_press_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
-{ 
+{
 	TotemPlaylist *playlist = (TotemPlaylist *)data;
 
 	if (playlist->priv->drop_disabled)
@@ -746,7 +747,7 @@ button_press_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 	return FALSE;
 }
 
-static gboolean 
+static gboolean
 button_release_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 {
 	TotemPlaylist *playlist = (TotemPlaylist *)data;
@@ -766,7 +767,7 @@ button_release_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 	return FALSE;
 }
 
-static void 
+static void
 drag_begin_cb (GtkWidget *treeview, GdkDragContext *context, gpointer data)
 {
 	TotemPlaylist *playlist = (TotemPlaylist *)data;
@@ -776,7 +777,7 @@ drag_begin_cb (GtkWidget *treeview, GdkDragContext *context, gpointer data)
 	return;
 }
 
-static void 
+static void
 drag_end_cb (GtkWidget *treeview, GdkDragContext *context, gpointer data)
 {
 	TotemPlaylist *playlist = (TotemPlaylist *)data;
@@ -1044,8 +1045,6 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 
 	response = gtk_dialog_run (GTK_DIALOG (fs));
 	gtk_widget_hide (fs);
-	while (gtk_events_pending())
-		gtk_main_iteration();
 
 	if (response == GTK_RESPONSE_ACCEPT) {
 		char *fname;
@@ -1469,7 +1468,7 @@ ensure_shuffled (TotemPlaylist *playlist)
 	} else {
 		current = -1;
 	}
-	
+
 	current_new = -1;
 
 	array = g_array_sized_new (FALSE, FALSE, sizeof (RandomData), PL_LEN);
@@ -1637,6 +1636,8 @@ totem_playlist_finalize (GObject *object)
 	if (playlist->priv->current != NULL)
 		gtk_tree_path_free (playlist->priv->current);
 
+	g_clear_pointer (&playlist->priv->tree_path, gtk_tree_path_free);
+
 	G_OBJECT_CLASS (totem_playlist_parent_class)->finalize (object);
 }
 
@@ -1747,6 +1748,7 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist,
 	char *filename_for_display, *uri, *escaped_filename;
 	GtkTreeRowReference *ref;
 	GFileMonitor *monitor;
+	GMount *mount;
 	GFile *file;
 	int pos;
 
@@ -1785,20 +1787,23 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist,
 				  "changed",
 				  G_CALLBACK (totem_playlist_file_changed),
 				  playlist);
+		mount = NULL;
 	} else {
+		mount = totem_get_mount_for_media (uri ? uri : mrl);
 		monitor = NULL;
 	}
 
 	escaped_filename = g_markup_escape_text (filename_for_display, -1);
 	gtk_list_store_insert_with_values (store, &iter, pos,
-			PLAYING_COL, TOTEM_PLAYLIST_STATUS_NONE,
-			FILENAME_COL, filename_for_display,
-			FILENAME_ESCAPED_COL, escaped_filename,
-			URI_COL, uri ? uri : mrl,
-			TITLE_CUSTOM_COL, display_name ? TRUE : FALSE,
-			FILE_MONITOR_COL, monitor,
-			MIME_TYPE_COL, content_type,
-			-1);
+					   PLAYING_COL, TOTEM_PLAYLIST_STATUS_NONE,
+					   FILENAME_COL, filename_for_display,
+					   FILENAME_ESCAPED_COL, escaped_filename,
+					   URI_COL, uri ? uri : mrl,
+					   TITLE_CUSTOM_COL, display_name ? TRUE : FALSE,
+					   FILE_MONITOR_COL, monitor,
+					   MOUNT_COL, mount,
+					   MIME_TYPE_COL, content_type,
+					   -1);
 	g_free (escaped_filename);
 
 	g_signal_emit (playlist,
@@ -2400,23 +2405,51 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 	playlist->priv->current_to_be_removed = FALSE;
 }
 
+static char *
+get_mount_default_location (GMount *mount)
+{
+	GFile *file;
+	char *path;
+
+	file = g_mount_get_root (mount);
+	if (file == NULL)
+		return NULL;
+	path = g_file_get_path (file);
+	g_object_unref (file);
+	return path;
+}
+
 static gboolean
 totem_playlist_compare_with_mount (TotemPlaylist *playlist, GtkTreeIter *iter, gconstpointer data)
 {
 	GMount *clear_mount = (GMount *) data;
-	char *mrl;
-
 	GMount *mount;
+	char *mount_path, *clear_mount_path;
 	gboolean retval = FALSE;
 
 	gtk_tree_model_get (playlist->priv->model, iter,
-			    URI_COL, &mrl, -1);
-	mount = totem_get_mount_for_media (mrl);
-	g_free (mrl);
+			    MOUNT_COL, &mount, -1);
 
-	if (mount == clear_mount)
+	if (mount == NULL)
+		return FALSE;
+
+	clear_mount_path = NULL;
+
+	mount_path = get_mount_default_location (mount);
+	if (mount_path == NULL)
+		goto bail;
+
+	clear_mount_path = get_mount_default_location (clear_mount);
+	if (clear_mount_path == NULL)
+		goto bail;
+
+	if (g_str_equal (mount_path, clear_mount_path))
 		retval = TRUE;
 
+bail:
+	g_free (mount_path);
+	g_free (clear_mount_path);
+
 	if (mount != NULL)
 		g_object_unref (mount);
 
@@ -2427,6 +2460,8 @@ void
 totem_playlist_clear_with_g_mount (TotemPlaylist *playlist,
 				   GMount *mount)
 {
+	g_return_if_fail (mount != NULL);
+
 	totem_playlist_clear_with_compare (playlist,
 					   (ClearComparisonFunc) totem_playlist_compare_with_mount,
 					   mount);
@@ -2465,10 +2500,10 @@ totem_playlist_get_current_mrl (TotemPlaylist *playlist, char **subtitle)
 }
 
 char *
-totem_playlist_get_current_title (TotemPlaylist *playlist, char **content_type)
+totem_playlist_get_current_title (TotemPlaylist *playlist)
 {
 	GtkTreeIter iter;
-	char *path;
+	char *title;
 
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), NULL);
 
@@ -2476,23 +2511,37 @@ totem_playlist_get_current_title (TotemPlaylist *playlist, char **content_type)
 		return NULL;
 
 	gtk_tree_model_get_iter (playlist->priv->model,
-			&iter,
-			playlist->priv->current);
+				 &iter,
+				 playlist->priv->current);
 
-	if (content_type != NULL) {
-		gtk_tree_model_get (playlist->priv->model,
-				    &iter,
-				    FILENAME_COL, &path,
-				    MIME_TYPE_COL, content_type,
-				    -1);
-	} else {
-		gtk_tree_model_get (playlist->priv->model,
-				    &iter,
-				    FILENAME_COL, &path,
-				    -1);
-	}
+	gtk_tree_model_get (playlist->priv->model,
+			    &iter,
+			    FILENAME_COL, &title,
+			    -1);
+	return title;
+}
 
-	return path;
+char *
+totem_playlist_get_current_content_type (TotemPlaylist *playlist)
+{
+	GtkTreeIter iter;
+	char *content_type;
+
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), NULL);
+
+	if (update_current_from_playlist (playlist) == FALSE)
+		return NULL;
+
+	gtk_tree_model_get_iter (playlist->priv->model,
+				 &iter,
+				 playlist->priv->current);
+
+	gtk_tree_model_get (playlist->priv->model,
+			    &iter,
+			    MIME_TYPE_COL, &content_type,
+			    -1);
+
+	return content_type;
 }
 
 char *
@@ -2635,7 +2684,7 @@ totem_playlist_set_playing (TotemPlaylist *playlist, TotemPlaylistStatus state)
 				      path, NULL,
 				      TRUE, 0.5, 0);
 	gtk_tree_path_free (path);
-	
+
 	return TRUE;
 }
 
@@ -2749,7 +2798,7 @@ totem_playlist_get_repeat (TotemPlaylist *playlist)
 
 	return playlist->priv->repeat;
 }
-	
+
 void
 totem_playlist_set_repeat (TotemPlaylist *playlist, gboolean repeat)
 {
@@ -2953,37 +3002,3 @@ totem_playlist_class_init (TotemPlaylistClass *klass)
 				totemplaylist_marshal_VOID__STRING_STRING,
 				G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_STRING);
 }
-
-static gboolean
-totem_playlist_foreach_cb (GtkTreeModel *model,
-			   GtkTreePath  *path,
-			   GtkTreeIter  *iter,
-			   gpointer      data)
-{
-	PlaylistForeachContext *context = data;
-	gchar *filename = NULL;
-	gchar *uri = NULL;
-
-	gtk_tree_model_get (model, iter, URI_COL, &uri, FILENAME_COL, &filename, -1);
-	context->callback (context->playlist, filename, uri, context->user_data);
-
-	g_free (filename);
-	g_free (uri);
-
-	return FALSE;
-}
-
-void
-totem_playlist_foreach (TotemPlaylist            *playlist,
-			TotemPlaylistForeachFunc  callback,
-			gpointer                  user_data)
-{
-	PlaylistForeachContext context = { playlist, callback, user_data };
-
-	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
-	g_return_if_fail (NULL != callback);
-
-	gtk_tree_model_foreach (playlist->priv->model,
-				totem_playlist_foreach_cb,
-				&context);
-}
